@@ -86,7 +86,7 @@ AnytoneInterface::RadioVariant::isValid() const {
  * Implementation of AnytoneInterface
  * ********************************************************************************************* */
 AnytoneInterface::AnytoneInterface(const USBDeviceDescriptor &descriptor, const ErrorStack &err, QObject *parent)
-  : USBSerial(descriptor, err, parent), _state(STATE_INITIALIZED), _info()
+  : USBSerial(descriptor, QSerialPort::Baud115200, err, parent), _state(STATE_INITIALIZED), _info()
 {
   if (isOpen()) {
     _state = STATE_OPEN;
@@ -107,7 +107,7 @@ AnytoneInterface::AnytoneInterface(const USBDeviceDescriptor &descriptor, const
 
 AnytoneInterface::~AnytoneInterface() {
   if (isOpen())
-    this->close();
+    AnytoneInterface::close();
 }
 
 USBDeviceInfo
@@ -116,8 +116,9 @@ AnytoneInterface::interfaceInfo() {
 }
 
 QList<USBDeviceDescriptor>
-AnytoneInterface::detect() {
-  return USBSerial::detect(USB_VID, USB_PID);
+AnytoneInterface::detect(bool saveOnly) {
+  Q_UNUSED(saveOnly);
+  return USBSerial::detect(USB_VID, USB_PID, true);
 }
 
 
