@@ -66,7 +66,7 @@ Codeplug::Element::fill(uint8_t value, unsigned offset, int size) {
 }
 
 bool
-Codeplug::Element::getBit(const Offset::BitOffset &offset) const {
+Codeplug::Element::getBit(const Offset::Bit &offset) const {
   return getBit(offset.byte, offset.bit);
 }
 
@@ -83,7 +83,7 @@ Codeplug::Element::getBit(unsigned offset, unsigned bit) const {
 }
 
 void
-Codeplug::Element::setBit(const Offset::BitOffset &offset, bool value) {
+Codeplug::Element::setBit(const Offset::Bit &offset, bool value) {
   setBit(offset.byte, offset.bit, value);
 }
 void
@@ -101,6 +101,10 @@ Codeplug::Element::setBit(unsigned offset, unsigned bit, bool value) {
     (*ptr) &= ~(1<<bit);
 }
 
+void
+Codeplug::Element::clearBit(const Offset::Bit &offset) {
+  clearBit(offset.byte, offset.bit);
+}
 void
 Codeplug::Element::clearBit(unsigned offset, unsigned bit) {
   if (offset >= _size) {
@@ -113,6 +117,16 @@ Codeplug::Element::clearBit(unsigned offset, unsigned bit) {
   (*ptr) &= ~(1<<bit);
 }
 
+
+uint8_t
+Codeplug::Element::getUInt2(const Offset::Bit &offset) const {
+  return getUInt2(offset.byte, offset.bit);
+}
+void
+Codeplug::Element::setUInt2(const Offset::Bit &offset, uint8_t value) {
+  setUInt2(offset.byte, offset.bit, value);
+}
+
 uint8_t
 Codeplug::Element::getUInt2(unsigned offset, unsigned bit) const {
   if (offset >= _size) {
@@ -135,6 +149,15 @@ Codeplug::Element::setUInt2(unsigned offset, unsigned bit, uint8_t value) {
   *(_data+offset) |= ((value & 0b11)<<bit);
 }
 
+uint8_t
+Codeplug::Element::getUInt3(const Offset::Bit &offset) const {
+  return getUInt3(offset.byte, offset.bit);
+}
+void
+Codeplug::Element::setUInt3(const Offset::Bit &offset, uint8_t value) {
+  setUInt3(offset.byte, offset.bit, value);
+}
+
 uint8_t
 Codeplug::Element::getUInt3(unsigned offset, unsigned bit) const {
   if (offset >= _size) {
@@ -157,6 +180,16 @@ Codeplug::Element::setUInt3(unsigned offset, unsigned bit, uint8_t value) {
   *(_data+offset) |= ((value & 0b111)<<bit);
 }
 
+uint8_t
+Codeplug::Element::getUInt4(const Offset::Bit &offset) const {
+  return getUInt4(offset.byte, offset.bit);
+}
+void
+Codeplug::Element::setUInt4(const Offset::Bit &offset, uint8_t value) {
+  setUInt4(offset.byte, offset.bit, value);
+}
+
+
 uint8_t
 Codeplug::Element::getUInt4(unsigned offset, unsigned bit) const {
   if (offset >= _size) {
@@ -179,6 +212,15 @@ Codeplug::Element::setUInt4(unsigned offset, unsigned bit, uint8_t value) {
   *(_data+offset) |= ((value & 0b1111)<<bit);
 }
 
+uint8_t
+Codeplug::Element::getUInt5(const Offset::Bit &offset) const {
+  return getUInt5(offset.byte, offset.bit);
+}
+void
+Codeplug::Element::setUInt5(const Offset::Bit &offset, uint8_t value) {
+  setUInt5(offset.byte, offset.bit, value);
+}
+
 uint8_t
 Codeplug::Element::getUInt5(unsigned offset, unsigned bit) const {
   if (offset >= _size) {
@@ -201,6 +243,15 @@ Codeplug::Element::setUInt5(unsigned offset, unsigned bit, uint8_t value) {
   *(_data+offset) |= ((value & 0b11111)<<bit);
 }
 
+uint8_t
+Codeplug::Element::getUInt6(const Offset::Bit &offset) const {
+  return getUInt6(offset.byte, offset.bit);
+}
+void
+Codeplug::Element::setUInt6(const Offset::Bit &offset, uint8_t value) {
+  setUInt6(offset.byte, offset.bit, value);
+}
+
 uint8_t
 Codeplug::Element::getUInt6(unsigned offset, unsigned bit) const {
   if (offset >= _size) {
@@ -601,7 +652,7 @@ Codeplug::Element::writeUnicode(unsigned offset, const QString &txt, unsigned ma
  * Implementation of CodePlug::Context
  * ********************************************************************************************* */
 Codeplug::Context::Context(Config *config)
-  : _config(config), _tables()
+  : _config(config), _satellites(nullptr), _tables()
 {
   // Add tables for common elements
   addTable(&DMRRadioID::staticMetaObject);
@@ -623,6 +674,11 @@ Codeplug::Context::config() const {
   return _config;
 }
 
+SatelliteDatabase *
+Codeplug::Context::satellites() const {
+  return _satellites;
+}
+
 bool
 Codeplug::Context::hasTable(const QMetaObject *obj) const {
   // Find a matching table
