@@ -1,19 +1,8 @@
+#include "userdatabase.hh"
 #include "d868uv_callsigndb.hh"
 #include "utils.hh"
 #include <QtEndian>
 
-#define MAX_CALLSIGNS               0x00030d40  // Maximum number of callsings in DB (200k)
-
-#define CALLSIGN_INDEX_BANK0        0x04000000  // Start address of the callsign index table
-#define CALLSIGN_INDEX_BANK_OFFSET  0x00040000  // Offset between index banks
-#define CALLSIGN_INDEX_BANK_SIZE    0x0001f400  // Size of each callsign index bank
-
-#define CALLSIGN_BANK0              0x04500000  // Start address of the actual call sign entries
-#define CALLSIGN_BANK_OFFSET        0x00040000  // Offset between callsign entry bank
-#define CALLSIGN_BANK_SIZE          0x000186a0  // Size of each callsign entry bank
-
-#define CALLSIGN_LIMITS             0x044C0000  // Start address of callsign db limits
-
 
 /* ********************************************************************************************* *
  * Implementation of D868UVCallsignDB::EntryElement
@@ -30,6 +19,12 @@ D868UVCallsignDB::EntryElement::EntryElement(uint8_t *ptr)
   // pass...
 }
 
+void
+D868UVCallsignDB::EntryElement::clear() {
+  // Clear header and all strings.
+  memset(_data, 0x00, 12);
+}
+
 void
 D868UVCallsignDB::EntryElement::setCallType(DMRContact::Type type) {
   switch (type) {
@@ -46,7 +41,7 @@ D868UVCallsignDB::EntryElement::setNumber(unsigned num) {
 
 void
 D868UVCallsignDB::EntryElement::setFriendFlag(bool set) {
-  setBit(0x0005, 3, set);
+  setBit(0x0005, 4, set);
 }
 
 void
@@ -61,7 +56,7 @@ D868UVCallsignDB::EntryElement::setContent(
 {
   unsigned addr = 0x0006;
   writeASCII(addr, name, 16, 0x00); addr += std::min(16, name.size()); setUInt8(addr, 0); addr++;
-  writeASCII(addr, city, 16, 0x00); addr += std::min(16, city.size()); setUInt8(addr, 0); addr++;
+  writeASCII(addr, city, 15, 0x00); addr += std::min(15, city.size()); setUInt8(addr, 0); addr++;
   writeASCII(addr, call, 8, 0x00); addr += std::min(8, call.size()); setUInt8(addr, 0); addr++;
   writeASCII(addr, state, 16, 0x00); addr += std::min(16, state.size()); setUInt8(addr, 0); addr++;
   writeASCII(addr, country, 16, 0x00); addr += std::min(16, country.size()); setUInt8(addr, 0); addr++;
@@ -70,6 +65,7 @@ D868UVCallsignDB::EntryElement::setContent(
 
 unsigned
 D868UVCallsignDB::EntryElement::fromUser(const UserDatabase::User &user) {
+  clear();
   setCallType(DMRContact::PrivateCall);
   setNumber(user.id);
   setRingTone(RingTone::Off);
@@ -81,7 +77,7 @@ unsigned
 D868UVCallsignDB::EntryElement::size(const UserDatabase::User &user) {
   return 6 // header
       + std::min(16, user.name.size())+1 // name
-      + std::min(16, user.city.size())+1 // city
+      + std::min(15, user.city.size())+1 // city
       + std::min( 8, user.call.size())+1 // call
       + std::min(16, user.state.size())+1 // state
       + std::min(16, user.country.size())+1 // country
@@ -130,7 +126,7 @@ D868UVCallsignDB::LimitsElement::setEndOfDB(unsigned addr) {
 }
 void
 D868UVCallsignDB::LimitsElement::setTotalSize(unsigned size) {
-  setEndOfDB(CALLSIGN_BANK0 + size);
+  setEndOfDB(D868UVCallsignDB::Offset::callsigns() + size);
 }
 
 unsigned
@@ -153,7 +149,7 @@ bool D868UVCallsignDB::encode(UserDatabase *db, const Selection &selection, cons
   Q_UNUSED(err)
 
   // Determine size of call-sign DB in memory
-  qint64 n = std::min(db->count(), qint64(MAX_CALLSIGNS));
+  qint64 n = std::min(db->count(), qint64(Limit::entries()));
   // If DB size is limited by settings
   if (selection.hasCountLimit())
     n = std::min(n, (qint64)selection.countLimit());
@@ -173,25 +169,25 @@ bool D868UVCallsignDB::encode(UserDatabase *db, const Selection &selection, cons
     dbSize += EntryElement::size(users[i]);
 
   // Allocate DB limits
-  image(0).addElement(CALLSIGN_LIMITS, LimitsElement::size());
-  memset(data(CALLSIGN_LIMITS), 0x00, LimitsElement::size());
+  image(0).addElement(Offset::limits(), LimitsElement::size());
   // Store DB limits
-  LimitsElement limits(data(CALLSIGN_LIMITS));
+  LimitsElement limits(data(Offset::limits()));
+  limits.clear();
   limits.setCount(n);
   limits.setTotalSize(dbSize);
 
   // Allocate index banks
-  for (int i=0; 0<indexSize; i++, indexSize-=std::min(indexSize, size_t(CALLSIGN_INDEX_BANK_SIZE))) {
-    size_t addr = CALLSIGN_INDEX_BANK0 + i*CALLSIGN_INDEX_BANK_OFFSET;
-    size_t size = align_size(std::min(indexSize, size_t(CALLSIGN_INDEX_BANK_SIZE)), 16);
+  for (int i=0; 0<indexSize; i++, indexSize-=std::min(indexSize, size_t(IndexBankElement::size()))) {
+    size_t addr = Offset::index() + i*Offset::betweenIndexBanks();
+    size_t size = align_size(std::min(indexSize, size_t(IndexBankElement::size())), 16);
     image(0).addElement(addr, size);
     memset(data(addr), 0xff, size);
   }
 
   // Allocate entry banks
-  for (int i=0; 0<dbSize; i++, dbSize-=std::min(dbSize, size_t(CALLSIGN_BANK_SIZE))) {
-    size_t addr = CALLSIGN_BANK0 + i*CALLSIGN_BANK_OFFSET;
-    size_t size = align_size(std::min(dbSize, size_t(CALLSIGN_BANK_SIZE)), 16);
+  for (int i=0; 0<dbSize; i++, dbSize-=std::min(dbSize, size_t(EntryBankElement::size()))) {
+    size_t addr = Offset::callsigns() + i*Offset::betweenCallsignBanks();
+    size_t size = align_size(std::min(dbSize, size_t(EntryBankElement::size())), 16);
     image(0).addElement(addr, size);
     memset(data(addr), 0x00, size);
   }
@@ -202,10 +198,10 @@ bool D868UVCallsignDB::encode(UserDatabase *db, const Selection &selection, cons
   uint32_t index_offset = 0;
   uint32_t index_bank   = 0;
   for (qint64 i=0; i<n; i++, index_offset+=IndexEntryElement::size()) {
-    if (CALLSIGN_INDEX_BANK_SIZE <= index_offset) {
+    if (IndexBankElement::size() <= index_offset) {
       index_offset = 0; index_bank += 1;
     }
-    IndexEntryElement index(data(CALLSIGN_INDEX_BANK0+index_bank*CALLSIGN_BANK_OFFSET+index_offset));
+    IndexEntryElement index(data(Offset::index()+index_bank*Offset::betweenIndexBanks()+index_offset));
     index.setID(users[i].id, false);
     index.setIndex(entry_offset);
     entry_offset += EntryElement::size(users[i]);
@@ -218,24 +214,26 @@ bool D868UVCallsignDB::encode(UserDatabase *db, const Selection &selection, cons
     // Get size of current entry
     uint32_t entry_size = EntryElement::size(users[i]);
     // Check if entry fits into bank
-    if (CALLSIGN_BANK_SIZE < (entry_offset+entry_size)) {
+    if (EntryBankElement::size() < (entry_offset+entry_size)) {
       // If not, split
       uint8_t buffer[100]; EntryElement(buffer).fromUser(users[i]);
-      uint32_t n1 = (CALLSIGN_BANK_SIZE-entry_offset);
+      uint32_t n1 = (EntryBankElement::size()-entry_offset);
       uint32_t n2 = entry_size-n1;
-      // Copy first half
-      uint8_t *ptr = data(CALLSIGN_BANK0+entry_bank*CALLSIGN_BANK_OFFSET+entry_offset);
-      memcpy(ptr, buffer, n1);
+      if (0 != n1) {
+        // Copy first half
+        uint8_t *ptr = data(Offset::callsigns()+entry_bank*Offset::betweenCallsignBanks()+entry_offset);
+        memcpy(ptr, buffer, n1);
+      }
       // advance bank counter
       entry_bank++; entry_offset = 0;
       // copy second half
-      ptr = data(CALLSIGN_BANK0+entry_bank*CALLSIGN_BANK_OFFSET+entry_offset);
+      uint8_t *ptr = data(Offset::callsigns()+entry_bank*Offset::betweenCallsignBanks()+entry_offset);
       memcpy(ptr, buffer+n1, n2);
       // Update offset
       entry_offset += n2;
     } else {
       // when it fits, just add
-      EntryElement(data(CALLSIGN_BANK0+entry_bank*CALLSIGN_BANK_OFFSET+entry_offset))
+      EntryElement(data(Offset::callsigns()+entry_bank*Offset::betweenCallsignBanks()+entry_offset))
           .fromUser(users[i]);
       entry_offset += entry_size;
     }
