@@ -16,13 +16,9 @@ EncryptionKey::clear() {
   _key.clear();
 }
 
-void
-EncryptionKey::fromHex(const QString &hex) {
-  QByteArray newKey = QByteArray::fromHex(hex.toLocal8Bit());
-  if (_key == newKey)
-    return;
-  _key = newKey;
-  emit modified(this);
+bool
+EncryptionKey::fromHex(const QString &hex, const ErrorStack &err) {
+  return setKey(QByteArray::fromHex(hex.toLocal8Bit()), err);
 }
 
 QString
@@ -35,19 +31,35 @@ EncryptionKey::key() const {
   return _key;
 }
 
+bool
+EncryptionKey::setKey(const QByteArray &key, const ErrorStack &err) {
+  if (key.isEmpty()) {
+    errMsg(err) << "Cannot set empty encryption key.";
+    return false;
+  }
+
+  if (_key == key)
+    return true;
+
+  _key = key;
+  emit modified(this);
+
+  return true;
+}
+
 
 /* ********************************************************************************************* *
- * Implementation of DMREncryptionKey
+ * Implementation of BasicEncryptionKey
  * ********************************************************************************************* */
-DMREncryptionKey::DMREncryptionKey(QObject *parent)
+BasicEncryptionKey::BasicEncryptionKey(QObject *parent)
   : EncryptionKey(parent)
 {
   // pass...
 }
 
 ConfigItem *
-DMREncryptionKey::clone() const {
-  DMREncryptionKey *key = new DMREncryptionKey();
+BasicEncryptionKey::clone() const {
+  BasicEncryptionKey *key = new BasicEncryptionKey();
   if (! key->copy(*this)) {
     key->deleteLater();
     return nullptr;
@@ -55,28 +67,84 @@ DMREncryptionKey::clone() const {
   return key;
 }
 
-void
-DMREncryptionKey::fromHex(const QString &hex) {
-  if (4 != hex.size()) {
-    logError() << "Cannot set DMR ecryption key to '" << hex << "': Not a 16bit key.";
-    return;
-  }
-  EncryptionKey::fromHex(hex);
-}
-
 YAML::Node
-DMREncryptionKey::serialize(const Context &context, const ErrorStack &err) {
+BasicEncryptionKey::serialize(const Context &context, const ErrorStack &err) {
   YAML::Node node = EncryptionKey::serialize(context, err);
   if (node.IsNull())
     return node;
 
   YAML::Node type;
-  type["basic"] = node;
+  type["dmr"] = node;
   return type;
 }
 
 bool
-DMREncryptionKey::parse(const YAML::Node &node, Context &ctx, const ErrorStack &err) {
+BasicEncryptionKey::parse(const YAML::Node &node, Context &ctx, const ErrorStack &err) {
+  if (! node)
+    return false;
+
+  if ((! node.IsMap()) || (1 != node.size())) {
+    errMsg(err) << node.Mark().line << ":" << node.Mark().column
+                << ": Cannot parse basic encryption key: Expected object with one child.";
+    return false;
+  }
+
+  YAML::Node key = node.begin()->second;
+  return EncryptionKey::parse(key, ctx, err);
+}
+
+
+/* ********************************************************************************************* *
+ * Implementation of EnhancedEncryptionKey
+ * ********************************************************************************************* */
+EnhancedEncryptionKey::EnhancedEncryptionKey(QObject *parent)
+  : EncryptionKey(parent)
+{
+  // pass...
+}
+
+ConfigItem *
+EnhancedEncryptionKey::clone() const {
+  EnhancedEncryptionKey *key = new EnhancedEncryptionKey();
+  if (! key->copy(*this)) {
+    key->deleteLater();
+    return nullptr;
+  }
+  return key;
+}
+
+bool
+EnhancedEncryptionKey::fromHex(const QString &hex, const ErrorStack &err) {
+  if (10 != hex.size()) {
+    errMsg(err) << "Cannot set RC4 (enhanced) ecryption key to '" << hex << "': Not a 40bit key.";
+    return false;
+  }
+  return EncryptionKey::fromHex(hex);
+}
+
+bool
+EnhancedEncryptionKey::setKey(const QByteArray &key, const ErrorStack &err) {
+  if (5 != key.size()) {
+    errMsg(err) << "Cannot set RC4 (enhanced) ecryption key: Not a 40bit key.";
+    return false;
+  }
+
+  return EncryptionKey::setKey(key, err);
+}
+
+YAML::Node
+EnhancedEncryptionKey::serialize(const Context &context, const ErrorStack &err) {
+  YAML::Node node = EncryptionKey::serialize(context, err);
+  if (node.IsNull())
+    return node;
+
+  YAML::Node type;
+  type["rc4"] = node;
+  return type;
+}
+
+bool
+EnhancedEncryptionKey::parse(const YAML::Node &node, Context &ctx, const ErrorStack &err) {
   if (! node)
     return false;
 
@@ -110,13 +178,13 @@ AESEncryptionKey::clone() const {
   return key;
 }
 
-void
-AESEncryptionKey::fromHex(const QString &hex) {
-  if (32 != hex.size()) {
-    logError() << "Cannot set AES ecryption key to '" << hex << "': Not a 16bit key.";
-    return;
+bool
+AESEncryptionKey::setKey(const QByteArray &key, const ErrorStack &err) {
+  if (16 > key.size()) {
+    errMsg(err) << "Cannot set AES ecryption key to '" << key.toHex() << "': Key smaller than 128bit.";
+    return false;
   }
-  EncryptionKey::fromHex(hex);
+  return EncryptionKey::setKey(key, err);
 }
 
 YAML::Node
@@ -150,18 +218,25 @@ AESEncryptionKey::parse(const YAML::Node &node, Context &ctx, const ErrorStack &
  * Implementation of EncryptionKeys
  * ********************************************************************************************* */
 EncryptionKeys::EncryptionKeys(QObject *parent)
-  : ConfigObjectList({DMREncryptionKey::staticMetaObject, AESEncryptionKey::staticMetaObject}, parent)
+  : ConfigObjectList({BasicEncryptionKey::staticMetaObject, EnhancedEncryptionKey::staticMetaObject, AESEncryptionKey::staticMetaObject}, parent)
 {
   // pass...
 }
 
 int
-EncryptionKeys::add(ConfigObject *obj, int row) {
+EncryptionKeys::add(ConfigObject *obj, int row, bool unique) {
   if ((nullptr == obj) || (! obj->is<EncryptionKey>())) {
     logError() << "Cannot add nullptr or non-encryption key objects to key list.";
     return -1;
   }
-  return ConfigObjectList::add(obj, row);
+  return ConfigObjectList::add(obj, row, unique);
+}
+
+EncryptionKey *
+EncryptionKeys::key(int index) const {
+  if (index >= count())
+    return nullptr;
+  return get(index)->as<EncryptionKey>();
 }
 
 ConfigItem *
@@ -177,8 +252,10 @@ EncryptionKeys::allocateChild(const YAML::Node &node, ConfigItem::Context &ctx,
   }
 
   QString type = QString::fromStdString(node.begin()->first.as<std::string>());
-  if ("basic" == type) {
-    return new DMREncryptionKey();
+  if (("basic" == type) || ("dmr" == type)) {
+    return new BasicEncryptionKey();
+  } else if ("rc4" == type) {
+    return new EnhancedEncryptionKey();
   } else if ("aes" == type) {
     return new AESEncryptionKey();
   }
