@@ -153,7 +153,7 @@ GD77Limits::GD77Limits(QObject *parent)
   add("commercial", new RadioLimitItem {
         {"encryptionKeys", new RadioLimitList(
          BasicEncryptionKey::staticMetaObject,
-         0, RadioddityCodeplug::EncryptionElement::Limit::basicEncryptionKeys(),
+         0, RadioddityCodeplug::EncryptionElement::Limit::keyCount(),
          new RadioLimitObject {
            {"name", new RadioLimitIgnored()},
            {"key", new RadioLimitStringRegEx("[0-9a-fA-F]{8}")}
