@@ -9,6 +9,8 @@
 #include "radioddity_interface.hh"
 #include "opengd77_interface.hh"
 #include "tyt_interface.hh"
+#include "dr1801uv_interface.hh"
+#include "c7000device.hh"
 
 
 /* ********************************************************************************************* *
@@ -41,15 +43,14 @@ USBDeviceInfo::USBDeviceInfo()
   // pass...
 }
 
-USBDeviceInfo::USBDeviceInfo(Class cls, uint16_t vid, uint16_t pid, bool save, bool identifiable)
-  : _class(cls), _vid(vid), _pid(pid), _save(save), _identifiable(identifiable)
+USBDeviceInfo::USBDeviceInfo(Class cls, uint16_t vid, uint16_t pid, bool save)
+  : _class(cls), _vid(vid), _pid(pid), _save(save)
 {
   // pass...
 }
 
 USBDeviceInfo::USBDeviceInfo(const USBDeviceInfo &other)
-  : _class(other._class), _vid(other._vid), _pid(other._pid), _save(other._save),
-    _identifiable(other._identifiable)
+  : _class(other._class), _vid(other._vid), _pid(other._pid), _save(other._save)
 {
   // pass...
 }
@@ -60,13 +61,15 @@ USBDeviceInfo::operator =(const USBDeviceInfo &other) {
   _vid = other._vid;
   _pid = other._pid;
   _save = other._save;
-  _identifiable = other._identifiable;
   return *this;
 }
 
 bool
 USBDeviceInfo::operator ==(const USBDeviceInfo &other) const {
-  return (other._class == _class) && (other._vid == _vid) && (other._pid == _pid);
+  // Class must match, VID/PID only need to match, if both are != 0.
+  return (other._class == _class) &&
+      ((! other.hasVendorID()) || (! hasVendorID()) || (other._vid == _vid))
+      && ((! other.hasProductID()) || (! hasProductID()) || (other._pid == _pid));
 }
 bool
 USBDeviceInfo::operator !=(const USBDeviceInfo &other) const {
@@ -87,10 +90,19 @@ USBDeviceInfo::interfaceClass() const {
   return _class;
 }
 
+bool
+USBDeviceInfo::hasVendorID() const {
+  return 0 != vendorId();
+}
 uint16_t
 USBDeviceInfo::vendorId() const {
   return _vid;
 }
+
+bool
+USBDeviceInfo::hasProductID() const {
+  return 0 != productId();
+}
 uint16_t
 USBDeviceInfo::productId() const {
   return _pid;
@@ -101,11 +113,6 @@ USBDeviceInfo::isSave() const {
   return _save;
 }
 
-bool
-USBDeviceInfo::isIdentifiable() const {
-  return _identifiable;
-}
-
 QString
 USBDeviceInfo::description() const {
   QString res;
@@ -123,6 +130,9 @@ USBDeviceInfo::description() const {
   case Class::HID:
     stream << "HID " << QString::number(_vid,16) << ":" << QString::number(_pid,16);
     break;
+  case Class::C7K:
+    stream << "C7000 " << QString::number(_vid,16) << ":" << QString::number(_pid,16);
+    break;
   }
   return res;
 }
@@ -189,6 +199,7 @@ USBDeviceDescriptor::isValid() const {
     return validSerial();
   case Class::DFU:
   case Class::HID:
+  case Class::C7K:
     return validRawUSB();
   }
   return false;
@@ -268,6 +279,9 @@ USBDeviceDescriptor::description() const {
   } else if (USBDeviceInfo::Class::HID == _class) {
     USBDeviceHandle addr = _device.value<USBDeviceHandle>();
     return QString("USB HID: bus %1, device %2").arg(addr.bus).arg(addr.device);
+  } else if (USBDeviceInfo::Class::C7K == _class) {
+    USBDeviceHandle addr = _device.value<USBDeviceHandle>();
+    return QString("USB C7000 HT: bus %1, device %2").arg(addr.bus).arg(addr.device);
   }
   return "Invalid";
 }
@@ -284,6 +298,7 @@ USBDeviceDescriptor::deviceHandle() const {
     break;
   case Class::DFU:
   case Class::HID:
+  case Class::C7K:
     return QString("%1:%2").arg(_device.value<USBDeviceHandle>().bus)
         .arg(_device.value<USBDeviceHandle>().device);
   case Class::Serial:
@@ -294,12 +309,14 @@ USBDeviceDescriptor::deviceHandle() const {
 }
 
 QList<USBDeviceDescriptor>
-USBDeviceDescriptor::detect() {
+USBDeviceDescriptor::detect(bool saveOnly) {
   QList<USBDeviceDescriptor> res;
-  res.append(AnytoneInterface::detect());
-  res.append(OpenGD77Interface::detect());
-  res.append(RadioddityInterface::detect());
-  res.append(TyTInterface::detect());
+  res.append(AnytoneInterface::detect(saveOnly));
+  res.append(OpenGD77Interface::detect(saveOnly));
+  res.append(RadioddityInterface::detect(saveOnly));
+  res.append(TyTInterface::detect(saveOnly));
+  res.append(DR1801UVInterface::detect(saveOnly));
+  res.append(C7000Device::detect(saveOnly));
   return res;
 }
 
