@@ -1,5 +1,6 @@
 #include "uv390_codeplug.hh"
 #include "logger.hh"
+#include "config.hh"
 #include "tyt_extensions.hh"
 #include <QTimeZone>
 
@@ -38,10 +39,6 @@
 #define ADDR_GPSSYSTEMS         0x03ec40
 #define GPSSYSTEM_SIZE          0x000010
 
-#define NUM_TEXTMESSAGES              50
-#define ADDR_TEXTMESSAGES       0x002180
-#define TEXTMESSAGE_SIZE        0x000120
-
 #define ADDR_EMERGENCY_SETTINGS 0x005a50
 #define NUM_EMERGENCY_SYSTEMS         32
 #define ADDR_EMERGENCY_SYSTEMS  0x005a60
@@ -189,6 +186,8 @@ UV390Codeplug::ChannelElement::toChannelObj(const ErrorStack &err) const {
     ex->enableAllowInterrupt(allowInterrupt());
     ex->enableDCDM(dualCapacityDirectMode());
     ex->enableDCDMLeader(dcdmLeader());
+    if (ch->is<DMRChannel>())
+      ex->setDMRSquelch(squelch());
   }
 
   return ch;
@@ -202,13 +201,12 @@ UV390Codeplug::ChannelElement::fromChannelObj(const Channel *chan, Context &ctx)
     setPower(ctx.config()->settings()->power());
   else
     setPower(chan->power());
-  setSquelch(0);
+  // By default, set to global default value.
+  setSquelch(ctx.config()->settings()->squelch());
 
   if (chan->is<FMChannel>()) {
     const FMChannel *achan = chan->as<const FMChannel>();
-    if (achan->defaultSquelch())
-      setSquelch(ctx.config()->settings()->squelch());
-    else
+    if (! achan->defaultSquelch())
       setSquelch(achan->squelch());
   }
 
@@ -219,6 +217,8 @@ UV390Codeplug::ChannelElement::fromChannelObj(const Channel *chan, Context &ctx)
     enableAllowInterrupt(ex->allowInterrupt());
     enableDualCapacityDirectMode(ex->dcdm());
     enableDCDMLeader(ex->dcdmLeader());
+    if (chan->is<DMRChannel>())
+      setSquelch(ex->dmrSquelch());
   }
 }
 
@@ -1005,11 +1005,23 @@ UV390Codeplug::clearMenuSettings() {
   MenuSettingsElement(data(ADDR_MENUSETTINGS)).clear();
 }
 
+
 void
 UV390Codeplug::clearTextMessages() {
-  memset(data(ADDR_TEXTMESSAGES), 0, NUM_TEXTMESSAGES*TEXTMESSAGE_SIZE);
+  MessageBankElement(data(Offset::messages())).clear();
 }
 
+bool
+UV390Codeplug::encodeTextMessages(Context &ctx, const Flags &flags, const ErrorStack &err) {
+  return MessageBankElement(data(Offset::messages())).encode(ctx, flags, err);
+}
+
+bool
+UV390Codeplug::decodeTextMessages(Context &ctx, const ErrorStack &err) {
+  return MessageBankElement(data(Offset::messages())).decode(ctx, err);
+}
+
+
 void
 UV390Codeplug::clearEmergencySystems() {
   EmergencySettingsElement(data(ADDR_EMERGENCY_SETTINGS)).clear();
