@@ -1,7 +1,6 @@
 #include "zone.hh"
 #include "channel.hh"
 #include "config.hh"
-#include "anytone_extension.hh"
 #include <QPushButton>
 #include <QHBoxLayout>
 #include <QVBoxLayout>
@@ -74,6 +73,11 @@ Zone::B() {
   return &_B;
 }
 
+bool
+Zone::contains(Channel *obj) const {
+  return _A.has(obj) || _B.has(obj);
+}
+
 AnytoneZoneExtension *
 Zone::anytoneExtension() const {
   return _anytone;
@@ -111,9 +115,9 @@ ZoneList::zone(int idx) const {
 }
 
 int
-ZoneList::add(ConfigObject *obj, int row) {
+ZoneList::add(ConfigObject *obj, int row, bool unique) {
   if (obj && obj->is<Zone>())
-    return ConfigObjectList::add(obj, row);
+    return ConfigObjectList::add(obj, row, unique);
   return -1;
 }
 
