@@ -5,35 +5,21 @@
 #include "errorstack.hh"
 #include <iostream>
 #include <QTest>
+#include "commercial_extension.hh"
 
 D578UVTest::D578UVTest(QObject *parent)
-  : QObject(parent)
+  : UnitTestBase(parent)
 {
   // pass...
 }
 
-void
-D578UVTest::initTestCase() {
-  ErrorStack err;
-  if (! _basicConfig.readYAML(":/data/config_test.yaml", err)) {
-    QFAIL(QString("Cannot open codeplug file: %1")
-          .arg(err.format()).toStdString().c_str());
-  }
-}
-
-void
-D578UVTest::cleanupTestCase() {
-  // clear codeplug
-  _basicConfig.clear();
-}
-
 void
 D578UVTest::testBasicConfigEncoding() {
   ErrorStack err;
   Codeplug::Flags flags; flags.updateCodePlug=false;
   D578UVCodeplug codeplug;
   if (! codeplug.encode(&_basicConfig, flags, err)) {
-    QFAIL(QString("Cannot encode codeplug for AnyTone AT-D578UV: {}")
+    QFAIL(QString("Cannot encode codeplug for AnyTone AT-D578UV: %1")
           .arg(err.format()).toStdString().c_str());
   }
 }
@@ -44,16 +30,39 @@ D578UVTest::testBasicConfigDecoding() {
   Codeplug::Flags flags; flags.updateCodePlug=false;
   D578UVCodeplug codeplug;
   if (! codeplug.encode(&_basicConfig, flags, err)) {
-    QFAIL(QString("Cannot encode codeplug for AnyTone AT-D578UV: {}")
+    QFAIL(QString("Cannot encode codeplug for AnyTone AT-D578UV: %1")
           .arg(err.format()).toStdString().c_str());
   }
 
   Config config;
   if (! codeplug.decode(&config, err)) {
-    QFAIL(QString("Cannot decode codeplug for AnyTone AT-D578UV: {}")
+    QFAIL(QString("Cannot decode codeplug for AnyTone AT-D578UV: %1")
           .arg(err.format()).toStdString().c_str());
   }
 }
 
+void
+D578UVTest::testChannelFrequency() {
+  ErrorStack err;
+  Codeplug::Flags flags; flags.updateCodePlug=false;
+  D578UVCodeplug codeplug;
+  codeplug.clear();
+  if (! codeplug.encode(&_channelFrequencyConfig, flags, err)) {
+    QFAIL(QString("Cannot encode codeplug for AnyTone D578UV: {}")
+          .arg(err.format()).toStdString().c_str());
+  }
+
+  Config config;
+  if (! codeplug.decode(&config, err)) {
+    QFAIL(QString("Cannot decode codeplug for AnyTone D578UV: %1")
+          .arg(err.format()).toStdString().c_str());
+  }
+
+  QCOMPARE(config.channelList()->channel(0)->rxFrequency(),
+           Frequency::fromHz(123456780ULL));
+  QCOMPARE(config.channelList()->channel(0)->txFrequency(),
+           Frequency::fromHz(999999990ULL));
+}
+
 QTEST_GUILESS_MAIN(D578UVTest)
 
